import sys
import QUB
import Buttons
import Dialogs
import os.path
import qubtree

from math import *

import win32com.client
from pythoncom import CoInitialize, CoUninitialize


def RowsAsNumArrays(stringRows):
    for row in stringRows:
        yield [float(x) for x in row.strip('\r\n').split('\t')]
        
def RowsAsEmpty(stringRows):
    for row in stringRows:
        yield (len(row.strip('\r\n')) == 0)

def SegmentSizes(stringRows):
    emptiness = RowsAsEmpty(stringRows)
    count = 0
    while True:
        while not emptiness.next():
            count += 1
        if count > 0:
            yield count
            count = 0

class QDFWriter:
    def __init__(self,qdfNode, sampling, scaling):
        self.root = qdfNode
        qdfNode['Sampling'].data = sampling
        qdfNode['Scaling'].data = scaling
        qdfNode['ADDataSize'].data = 2
        qdfNode['ADDataType'].data = 7
        self.sampling = sampling
        self.scaling = scaling
        self.seg = qubtree.NullNode()
        self.totalpoints = 0
    def beginSeg(self, count):
        try:
            self.seg.find('Channels').data.unloadRows()
        except:
            pass
        self.count = count
        self.seg = self.root['Segments'].insert('Segment', self.seg)
        self.seg['StartTime'].data = self.sampling * self.totalpoints
        self.segpoints = 0
    def write(self, rows):
        chan = self.seg['Channels'].data
        chan.preload = False
        if self.segpoints == 0:
            cols = len(rows[0])
            self.root['ADChannelCount'].data = cols
            chan.setup(7, self.count, len(rows[0]))
        chan.loadRows(self.segpoints, self.segpoints+len(rows)-1, False)
        scal = self.scaling
        for i, row in enumerate(rows):
            outrow = chan.row(self.segpoints+i)
            for j, num in enumerate(row):
                outrow[j] = int(round(scal * num))
        self.segpoints += len(rows)
        self.totalpoints += len(rows)

MAXPAGE = 1 << 16
        
def TextToQDF(inpath, outpath, sampling, scaling, timeCol, msgOut):
    segSizes = [x for x in SegmentSizes(open(inpath))]
    if not segSizes:
        msgOut('Can\'t read %s' % inpath)
        return
    qdf = qubtree.Node('DataFile')
    if not qdf.saveAs(outpath):
        msgOut('Can\'t save %s' % outpath)
        return
    writer = QDFWriter(qdf, sampling, scaling)
    reader = RowsAsNumArrays(open(inpath))
    if timeCol:
        process = lambda row: row[1:]
    else:
        process = lambda row: row
    for count in segSizes:
        writer.beginSeg(count)
        while count > 0:
            rows = []
            for i in xrange(min(count, MAXPAGE)):
                rows.append(process(reader.next()))
            count -= MAXPAGE
            writer.write(rows)
    del writer
    qdf.save()

def ImportText(props):
    wiz = QUB.CreateWizard()
    wiz.caption = 'Import text'
    wiz.message = 'Converting text to qdf'
    wiz.buttons = []
    wiz.show()
    def msgOut(msg):
        wiz.message = msg
    TextToQDF(props['inpath'], props['outpath'], 1.0/(props['sampling']*1.0e3), props['scaling'], props['timeCol'], msgOut)
    


Buttons.AddTaskButton( 'Imprt Txt', # button text
                       ImportText,   # function
                       'ImpTxt',       # short text for small buttons
                       1,            # show settings dialog on left-click?
                       1,            # show settings dialog on right-click?
                       "Convert a text file into QUB data",   # tool-tip
                       "Convert a text file into QUB data",   # description in "Edit Buttons"
                       "Import txt data:", # caption for the settings dialog
                       [Dialogs.DialogText("inpath", ".txt file:", "", Dialogs.acceptString),
                        Dialogs.DialogText("outpath", ".qdf file:", "", Dialogs.acceptString),
                        Dialogs.DialogText("sampling", "sampling(kHz):", 20.0, Dialogs.acceptFloatGreaterThan(0.0)),
                        Dialogs.DialogText("scaling", "scaling:", 320, Dialogs.acceptIntGreaterThan(0)),
                        Dialogs.DialogCheckBox("timeCol", "Time column", False)
                        ],
                       'Scripts' )
